{#############################################################################

P5 DataGlove Component for Delphi

P5 is a registered Trademark of Essential Reality LLC
http://www.essentialreality.com

Author:       Andreas Ralf Schwarz (andreas.schwarz@hamburg.de)
Description:  P5 DataGlove Component for Delphi
Creation:     November 27, 2002
Version:      1.25
EMail:        andreas.schwarz@hamburg.de
Support:      No support yet
Legal issues: Copyright (C) 2002 Andreas Schwarz
              P5: Copyright (c) 2001 Essential Reality LLC

              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages arising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

              4. You must register this software (registering is free) :

                 http://www.dieknueppelkuh.de/p5glove/register.php

History:
06. Dec 2002     * Optimized Code
02. Dec 2002     Updated C++ Wrapper DLL, Motion and Bend is more smoother
                 * Added PositionOptimized Properties: Relative and Absolute
                 * Changed Wrapper DLL Structure
28. Nov 2002     C++ Wrapper DLL does work well with Delphi

#############################################################################}

unit DataGlove;

interface


uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

const
        // Max Bend
        P5BEND_MAX              = 63;

        // Dynamic Link Library Name
        P5Wrapper = 'P5WRAPPER.DLL';

        // Glove types
        P5GLOVETYPE_RIGHTHAND	       = $00;
        P5GLOVETYPE_LEFTHAND	       = $01;

        // Axis
        P5MOTION_NORMALAXIS	       = 1;
        P5MOTION_INVERTAXIS	       = -1;

type
        // P5ButtonData, gives the state of the buttons
        PP5ButtonData = ^TP5ButtonData;
        TP5ButtonData = record
         ButtonA, ButtonB, ButtonC, ButtonD : Boolean;
        end;

        PP5BendData = ^TP5BendData;
        TP5BendData = record
         Thumb, Index, Middle, Ring, Pinky : Byte;
        end;

        PP5PositionData = ^TP5PositionData;
        TP5PositionData = record
 	 PosX, PosY, PosZ, Yaw, Pitch, Roll : Single;
        end;

        TP5Region = record
         StartX, EndX,
         StartY, EndY,
         StartZ, EndZ : Integer;
        end;

        TP5GloveType = (gtLeftHand, gtRightHand);

        TP5DataGlove = class(TComponent)
         private
          FTimer : TTimer;
          FInterval : Integer;
          FInitialized : Boolean;
          FThumb   : Byte;
          FIndex   : Byte;
          FMiddle  : Byte;
          FRing    : Byte;
          FPinky   : Byte;
          FButtonA  : Boolean;
          FButtonB  : Boolean;
          FButtonC  : Boolean;
          FButtonD  : Boolean;
          FOnTimer : TNotifyEvent;
          FPosition : TP5PositionData;
          FPositionOptimizedAbsolute : TP5PositionData;
          FPositionOptimizedRelative : TP5PositionData;
          FGloveType : TP5GloveType;
          procedure DoInternalTimer(Sender : TObject);
          procedure SetInterval(Value : Integer);
         published
          property Interval : Integer read FInterval write SetInterval;
          property FingerThumb : Byte read FThumb;
          property FingerIndex : Byte read FIndex;
          property FingerMiddle : Byte read FMiddle;
          property FingerRing : Byte read FRing;
          property FingerPinky : Byte read FPinky;
          property ButtonA : Boolean read FButtonA;
          property ButtonB : Boolean read FButtonB;
          property ButtonC : Boolean read FButtonC;
          property ButtonD : Boolean read FButtonD;
          property Position : TP5PositionData read FPosition;
          property PositionOptimizedAbsolute : TP5PositionData read FPositionOptimizedAbsolute;
          property PositionOptimizedRelative : TP5PositionData read FPositionOptimizedRelative;
          property OnProcess : TNotifyEvent read FOnTimer write FOnTimer;
          property GloveType : TP5GloveType read FGloveType write FGloveType;
         public
          constructor Create(AOwner : TComponent); override;
          destructor Destroy; override;
          procedure GetButtonState;
          procedure GetPosition;
          procedure GetBendState;
          procedure ResetPosition;
          procedure UpdateInvertAxes(xAxis, yAxis, zAxis : Boolean);
          procedure UpdateRegion(StartX, EndX, StartY, EndY, StartZ, EndZ : Integer);
        end;


        procedure Register;

implementation

{$R *.dcr}

function P5BendState(BendData : PP5BendData) : Integer; stdcall; external P5Wrapper index 1;
function P5ButtonState(ButtonData : PP5ButtonData) : Integer; stdcall; external P5Wrapper index 2;
function P5FreeDevice : Boolean; stdcall; external P5Wrapper index 3;
function P5InitializeDevice : Boolean; stdcall; external P5Wrapper index 4;
function P5InvertMouse(xaxis, yaxis, zaxis : Integer) : Integer; stdcall; external P5Wrapper index 5;
function P5IsDevicePresent : Boolean; stdcall; external P5Wrapper index 6;
function P5Position(PosNormal : PP5PositionData; PosOptAbs : PP5PositionData; PosOptRel : PP5PositionData) : Integer; stdcall; external P5Wrapper index 7;
function P5SetClipRegion(xstart, xend, ystart, yend, zstart, zend : Integer) : Integer; stdcall; external P5Wrapper index 8;

{ TP5DataGlove }
constructor TP5DataGlove.Create(AOwner : TComponent);
begin
        inherited Create(AOwner);

        FInterval := 1;

        FTimer := TTimer.Create(Self);
        FTimer.OnTimer := DoInternalTimer;
        FTimer.Interval := FInterval;
        FTimer.Enabled  := False;

        FInitialized := P5InitializeDevice;
        FTimer.Enabled := True;
        ResetPosition;
end;

destructor TP5DataGlove.Destroy;
begin
        P5FreeDevice;
        inherited Destroy;
end;

procedure TP5DataGlove.SetInterval(Value : Integer);
begin
        if Value <> FInterval then FTimer.Interval := Value;                        
end;

procedure TP5DataGlove.DoInternalTimer(Sender : TObject);
begin
        GetButtonState;
        GetPosition;
        GetBendState;

        if Assigned(FOnTimer) then FOnTimer(Self);
end;

procedure TP5DataGlove.GetButtonState;
var ButtonState : TP5ButtonData;
begin
        P5ButtonState(@ButtonState);
        FButtonA := ButtonState.ButtonA;
        FButtonB := ButtonState.ButtonB;
        FButtonC := ButtonState.ButtonC;
        FButtonD := ButtonState.ButtonD;
end;

procedure TP5DataGlove.UpdateInvertAxes(xAxis, yAxis, zAxis : Boolean);
var x, y, z : Integer;
begin
        if XAxis then X := P5MOTION_NORMALAXIS else X := P5MOTION_INVERTAXIS;
        if YAxis then Y := P5MOTION_NORMALAXIS else Y := P5MOTION_INVERTAXIS;
        if ZAxis then Z := P5MOTION_NORMALAXIS else Z := P5MOTION_INVERTAXIS;

        P5InvertMouse(x, y, z);
end;        

procedure TP5DataGlove.UpdateRegion(StartX, EndX, StartY, EndY, StartZ, EndZ : Integer);
begin
        P5SetClipRegion(StartX, EndX,
                        StartY, EndY,
                        StartZ, EndZ);
end;

procedure TP5DataGlove.ResetPosition;
begin
      
end;

procedure TP5DataGlove.GetPosition;
begin
        P5Position(@FPosition, @FPositionOptimizedAbsolute, @FPositionOptimizedRelative);
end;

procedure TP5DataGlove.GetBendState;
var BendData : TP5BendData;
        procedure SetFinger(var Finger : Byte; Value : Integer);
         begin  Finger := P5BEND_MAX - Value; end;
begin
        P5BendState(@BendData);

        SetFinger(FThumb, BendData.Thumb);
        SetFinger(FIndex, BendData.Index);
        SetFinger(FMiddle, BendData.Middle);
        SetFinger(FRing, BendData.Ring);
        SetFinger(FPinky, BendData.Pinky);
end;

{ Register }
procedure Register;
begin
  RegisterComponents('P5 DataGlove', [TP5DataGlove]);
end;

end.



